# =========================================================================== #
# Gemspec for Project AsciiParadise.
# =========================================================================== #
require 'ascii_paradise'
require 'roebe'

Gem::Specification.new { |s|

  s.name    = 'ascii_paradise'
  s.version = AsciiParadise::VERSION
  s.date    = Time.now.strftime('%Y-%m-%d')
  
  DESCRIPTION = <<-EOF

This gem is called ascii_paradise.

AsciiParadise is an umbrella project that allows us to gather
ascii-related code, in Ruby.

The main focus for this project is to incorporate animated
ASCII components, but the project will also store some
non-animated ASCII - thus, essentially making it an umbrella
project for (useful) ASCII content. The main focus will
remain on animated ASCII, though.

Presently, #{AsciiParadise.n_animated_ascii_components}
ASCII components are registered in this project.

For more information about this project, have a look
at the documentation of this gem's homepage.

  EOF

  s.summary     = DESCRIPTION
  s.description = DESCRIPTION

  s.extra_rdoc_files = %w()
  
  s.authors  = ['Robert A. Heiler']
  s.email    = Roebe.email?
  s.files    = Dir['**/*']
  s.license  = 'MIT'
  s.homepage = 'https://rubygems.org/gems/ascii_paradise'

  # ========================================================================= #
  # Show this when a user installs this project.
  # ========================================================================= #
  s.post_install_message = <<-EOF

You can test this in your project through:

    require 'ascii_paradise'

    AsciiParadise.ascii_counter 20 # Start to countdown from 20.
    AsciiParadise.random

Or:

    AsciiParadise.say 'I will summon a dragon.', :wizard
    AsciiParadise.say 'Where is the dragon!', :knight
    AsciiParadise.say 'I am the cow!', :cowsay

Note that the last is the well-known cowsay program.

You can also pass --debug to the individual ascii components
and --frame to show the frame; and last but not least, also
--everything.

As of March 2016, you can also invoke the various components
based on their file name.

Examples:

    ascii_paradise Volcano
    ascii_paradise --endless_runner
    ascii_paradise --diver
    ascii_paradise --moth
    ascii_paradise --traffic

And so on.

Note that as of March 2017, this requires an upcased name
such as Volcano. At a later time, we will map a downcased
name to the name of the constant - and get rid of this
notification here as well.

You can also show individual frames and then colourize them
through the pipe if you are on Linux and use the KDE Konsole.

Example for this:

  asciiparadise tetris --show-frame=5 | rainbow_colours

For more documentation, have a look at:

  #{AsciiParadise::URL_TO_THE_DOCUMENTATION}

  EOF

  s.required_ruby_version     = '>= '+Roebe.third_most_stable_version_of_ruby
  s.required_rubygems_version = '>= '+Gem::VERSION
  s.rubygems_version          = '>= '+Gem::VERSION

  # ========================================================================= #
  # External dependencies for the project:
  # ========================================================================= #
  s.add_dependency 'colours'
  s.add_dependency 'opn'

}