#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
require 'ascii_paradise'
require 'ascii_paradise/asciitable/autoinclude'
require 'colours/autoinclude'

@counter = 0

# =========================================================================== #
# === orange
# =========================================================================== #
def orange(i)
  Colours.orange(i)
end

# =========================================================================== #
# === steelblue
# =========================================================================== #
def steelblue(i)
  Colours.steelblue(i)
end

# =========================================================================== #
# === bold_blue
# =========================================================================== #
def bold_blue(i)
  Colours.royalblue(i)
end

# =========================================================================== #
# === example
# =========================================================================== #
def example
  @counter += 1
  e "\n"+orange('Example ')+bold_blue(@counter.to_s)+' '+
     steelblue('-' * 60)
  e
end

# =========================================================================== #
# === Example 1
#
# Use the table() method.
# =========================================================================== #
example
array = [['header left', 'header right'], [1, 2], [3, 4]]
e table(*array)
e

# =========================================================================== #
# Example 2 tag - in this example, we use padding.
# =========================================================================== #
example
e "This table also shows #{steelblue(':width')} and "\
  "#{steelblue(':padding_left')} settings, and a "\
  "manually added separator."
e
t = table ['header left', 'header right']
t.style = {:padding_left => 1, :width => 80}
t << [1, 2]
t << [3, 4]
t.add_separator
t << [4, 6]
e t

# =========================================================================== #
# Example 3 tag
# =========================================================================== #
example
e 'The next example will also show .title='
e
user_table = table { |ascii_table| # Add in block format.
  ascii_table.set_title 'Diamond Help'
  ascii_table.headings = 'Left', 'Right'
  ascii_table << %w( Yoo there )
  ascii_table << [sfancy('Ok'), 'man']
  ascii_table << ['Joe', bold_blue('yooo')]
  ascii_table << ['mouse:', 'Jerry']
  ascii_table << ['cat:',   'Tom']
}
e user_table

# =========================================================================== #
# Example 4 tag
# =========================================================================== #
example
e 'Next we will use three headings:'
e
user_table = table { |v| # Add in block format.
  v.header = 'Contact Information'
  v.headings = 'First Name', 'Last Name', 'Email'
  v << %w( TJ  Holowaychuk tj@vision-media.ca )
  v << %w( Bob Someone     bob@vision-media.ca )
  v << %w( Joe Whatever    bob@vision-media.ca )
}
e user_table
e
# =========================================================================== #
# Example 5 tag
# =========================================================================== #
example
e 'This variant will explicitely use a width of 80'
e
user_table = table { |v|
  v.style.width = 80
  v.headings = 'First Name', 'Last Name', 'Email'
  v << %w( TJ Holowaychuk tj@vision-media.ca )
  v << %w( Bob Someone bob@vision-media.ca )
  v << %w( Joe Whatever bob@vision-media.ca )
}
e user_table

# =========================================================================== #
# Example 6 tag.
# =========================================================================== #
example
user_table = table {
  self.headings = 'First Name', 'Last Name', 'Email'
  add_row ['TJ',  'Holowaychuk', 'tj@vision-media.ca']
  add_row ['Bob', 'Someone',     'bob@vision-media.ca']
  add_row ['Joe', 'Whatever',    'joe@vision-media.ca']
  add_separator
  add_row ['Total', { :value => '3', :colspan => 2, alignment: :middle }]
  align_column 1, :center
}
e user_table
e
# =========================================================================== #
# Example 7 tag
# =========================================================================== #
example
e 'This variant makes use of '+steelblue('add_row()')+' which '\
  'will also span two columns.'
e
user_table = table {
  self.headings = ['First Name', 'Last Name', {:value => 'Phones', :colspan => 2, :alignment => :center}]
  add_row ['Bob', 'Someone',     '123', '456']
  add :separator
  add_row ['TJ',  'Holowaychuk', {:value => "No phones\navaiable", :colspan => 2, :alignment => :center}]
  add :separator
  add_row ['Joe', 'Whatever',    '4324', '343242']
}
e user_table

# =========================================================================== #
# Example 8 tag - this one is simpler.
# =========================================================================== #
example
AsciiParadise::AsciiTable.enable_colours
rows = []
rows << ['Lines',      100]
rows << ['Comments',   20]
rows << ['Ruby',       70]
rows << ['JavaScript', 30]
e table([nil, 'Lines'], *rows) # We simply push the Array into this method.

rows = []
rows << ['Lines',      100]
rows << ['Comments',   20]
rows << ['Ruby',       70]
rows << ['JavaScript', 30]
e table(nil, *rows)
e
e 'Next, we will align towards the right side.'
e
rows = []
rows << ['Lines',      100]
rows << ['Comments',    20]
rows << ['Ruby',        70]
rows << ['JavaScript',  30]
table = table([{ :value => 'Stats', :colspan => 2, :alignment => :center }], *rows)
table.align_column 1, :right
e table

AsciiParadise::AsciiTable.use_this_colour :lightblue
e
# =========================================================================== #
# Example 9 tag
# =========================================================================== #
example
e
require 'ascii_paradise/animations/tetris.rb'
array = AsciiParadise.return_dataset_for(:tetris)
user_table = table { |v|
  v.style.width = 88
  v.headings = 'First Name', 'Last Name', 'Email'
  inner_array = [array[3], array[4], array[5]]
  v << inner_array
  v << %w( Bob Someone bob@vision-media.ca )
  v << %w( Joe Whatever bob@vision-media.ca )
}
e user_table
# $RSRC/asciitable/samples/all_in_one_sample.rb